/*
 * Project name:
     Virtual COM Port Demo
 * Description
     Example showing usage USB CDC device that functions as a generic virtual 
     COM port. The example echoes data sent via USART terminal.
 * Test configuration:
     MCU:             P32MX795F512L
     dev.board:       EasyPIC Fusion v7
                      http://www.mikroe.com/easypic-fusion/
     Oscillator:      XT-PLL, 80.000MHz
     Ext. Modules:    None.
     SW:              mikroC PRO for PIC32
                      http://www.mikroe.com/mikroc/pic32/
     Notes:           Example uses CDC Driver included in Windows. An .inf file
                      is supplied with the example in order to match the driver
                      with this device (user must guide driver installation 
                      wizard to the VCPDriver folder).

                      Turn on USB VBUS switch.
 */

#include <stdint.h>
#include "IHM.h"

// Buffer of 64 bytes
char buffer[64];
char txt_line[64] = "";
char short_value[20] = "";
uint32_t baudrate = 19200;
uint8_t stop_bit = 0;
uint8_t parity = 0;
uint8_t data_bits = 8;
void ConfigUART ();

// Line coding structure
typedef struct{
  uint32_t dwDTERate;   // Data terminal rate, in bits per second
  uint8_t bCharFormat;  // Stop bits 0 - 1 Stop bit
                        //           1 - 1.5 Stop bits
                        //           2 - 2 Stop bits
  uint8_t bParityType;  // Parity 0 - None
                        //        1 - Odd
                        //        2 - Even
                        //        3 - Mark
                        //        4 - Space
  uint8_t bDataBits;    // Data bits (5, 6, 7, 8 or 16)

}TUSBCDCLineCoding;

TUSBCDCLineCoding * COM_Conf;

void USB1Interrupt() iv IVT_USB_1 ilevel 7 ics ICS_SRS{
  // Call library interrupt handler routine
  USBDev_IntHandler();
}

void UART1_Isr() iv IVT_UART_1 ilevel 6 ics ICS_SOFT {
     int j;
     
     j = 0;

     while (UART1_Data_Ready()) {
           *(buffer+j++) = UART1_Read();
     }

     USBDev_CDCSendData(buffer, j);
     
     U1RXIF_bit = 0;
}

void USBDev_CDCParamsChanged(){
  ConfigUART ();
}

extern const uint8_t _USB_CDC_BULK_EP_IN;   // Data interface IN endpoint
extern const uint8_t _USB_CDC_BULK_EP_OUT;  // Data interface OUT endpoint

void USBDev_CDCDataReceived(uint16_t size){
  int i;
  
  i = 0;
  // Prepare receive buffer
  USBDev_CDCSetReceiveBuffer(buffer);
  // Send back received packet
  for (i=0; i<size; i++) UART1_Write(*(buffer+i));
  //USBDev_CDCSendData(buffer, size);
}

void ConfigUART () {
     baudrate =  COM_Conf->dwDTERate;
     /*
     IHM_ClearSreen();
     sprintf(txt_line , "Baudrate: %lu", baudrate);
     IHM_Write_Line (txt_line);
     switch (COM_Conf->bCharFormat) {
            case 0:
                 stop_bit = _UART_ONE_STOPBIT;
                 strcpy(short_value, "1 bit");
                 break;
            case 2:
                 stop_bit = _UART_TWO_STOPBITS;
                 strcpy(short_value, "2 bits");
                 break;
            default:
                 stop_bit = _UART_ONE_STOPBIT;
                 strcpy(short_value, "1 bit");
                 break;
     }
     sprintf(txt_line , "Stop bits: %s", short_value);
     IHM_Write_Line (txt_line);
    if (COM_Conf->bDataBits == 8) {
       switch (COM_Conf->bParityType) {
              case 0:
                   parity = _UART_8BIT_NOPARITY;
                   strcpy(short_value, "No");
                   break;
              case 1:
                   parity = _UART_8BIT_ODDPARITY;
                   strcpy(short_value, "Odd");
                   break;
              case 2:
                   parity = _UART_8BIT_EVENPARITY;
                   strcpy(short_value, "Even");
                   break;
              default:
                   parity = _UART_8BIT_NOPARITY;
                   strcpy(short_value, "No");
                   break;
       }
       sprintf(txt_line , "Parity: %s", short_value);
       IHM_Write_Line (txt_line);
       strcpy(short_value, "8 bits");
       sprintf(txt_line , "Data bits: %s", short_value);
       IHM_Write_Line (txt_line);
    }
    else if (COM_Conf->bDataBits == 9) {
         parity = _UART_9BIT_NOPARITY;
         strcpy(short_value, "No");
         sprintf(txt_line , "Parity: %s", short_value);
         IHM_Write_Line (txt_line);
         strcpy(short_value, "9 bits");
         sprintf(txt_line , "Data bits: %s", short_value);
    }
    else {
         parity = _UART_8BIT_NOPARITY;
         strcpy(short_value, "No");
         sprintf(txt_line , "Parity: %s", short_value);
         IHM_Write_Line (txt_line);
         strcpy(short_value, "8 bits");
         sprintf(txt_line , "Data bits: %s", short_value);
    }
    
    UART1_Init_Advanced(baudrate, 80000, _UART_HIGH_SPEED , parity, stop_bit);
    */
    UART1_Init(baudrate);                 // Wait for UART module to stabilize
}

void main() {
  uint32_t old_baudrate = 0;
  trisd.f11 = 0;

  // Reset Parani ESD 110
  latd.f11 = 0;
  Delay_ms(1500);
  latd.f11 = 1;
  
  AD1PCFG = 0xFFFF;                        // Configure AN pins as digital I/O

  // Initialize CDC Class
  USBDev_CDCInit();
  
  // Initialize USB device module
  USBDev_Init();
 
  // Enable USB device interrupt
  USBIE_bit = 1;
  IPC11bits.USBIP = 7;
  U1IP0_bit = 0;
  U1IP1_bit = 1;
  U1IP2_bit = 1;
  U1RXIE_bit = 1;
  EnableInterrupts();

  // Wait until device is configured (enumeration is successfully finished)
  while(USBDev_GetDeviceState() != _USB_DEV_STATE_CONFIGURED)
    ;
 
  // Set receive buffer where received data is stored
  USBDev_CDCSetReceiveBuffer(buffer);

  COM_Conf = USBDev_CDCGetLineCoding();
  
  IHM_Init ();
  ConfigUART ();

  // Infinite loop
  while(1){
     if(old_baudrate != baudrate) {
       IHM_ClearSreen();
       baudrate =  COM_Conf->dwDTERate;
       old_baudrate = baudrate;
       sprintf(txt_line , "Baudrate: %lu", baudrate);
       IHM_Write_Line (txt_line);
     }
  }

}